from typing import *
import royalnet.commands as rc
import royalnet.backpack.tables as rbt

from ..tables import FiorygiTransaction


class MagickfiorygiCommand(rc.Command):
    name: str = "magickfiorygi"

    description: str = "Crea fiorygi dal nulla."

    syntax: str = "{destinatario} {quantità} {motivo}"

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        async with data.session_acm() as session:
            author = await data.find_author(session=session, required=True)
            if "banker" not in author.roles:
                raise rc.UserError("Non hai permessi sufficienti per eseguire questo comando.")

            user_arg = args[0]
            qty_arg = args[1]
            reason_arg = " ".join(args[2:])

            if user_arg is None:
                raise rc.InvalidInputError("Non hai specificato un destinatario!")
            user = await rbt.User.find(self.alchemy, session, user_arg)
            if user is None:
                raise rc.InvalidInputError("L'utente specificato non esiste!")

            if qty_arg is None:
                raise rc.InvalidInputError("Non hai specificato una quantità!")
            try:
                qty = int(qty_arg)
            except ValueError:
                raise rc.InvalidInputError("La quantità specificata non è un numero!")

            if reason_arg == "":
                raise rc.InvalidInputError("Non hai specificato un motivo!")

            await FiorygiTransaction.spawn_fiorygi(user, qty, reason_arg, data=data, session=session)



from typing import *
from .trionfoinfo import TrionfoInfo
from .check import *

trionfilist: List[TrionfoInfo] = [
    TrionfoInfo(
        variable="zero",
        title="o",
        roman="0",
        name="Il Folle",
        puzzle="UN VIAGGIO TI ATTENDE",
        objective="Trova la pagina dei Trionfi Reali.",
        check=NullCheck(),
    ),
    TrionfoInfo(
        variable="i",
        title="i",
        roman="I",
        name="Il Mago",
        puzzle="L'ULTIMO GIORNO",
        objective="Gioca almeno mezz'ora a [url=https://store.steampowered.com/app/42910]Magicka[/url] "
                  "o [url=https://store.steampowered.com/app/238370]Magicka 2[/url].",
        check=CheckPlayedSteamGame(42910) or CheckPlayedSteamGame(238370),
    ),
    TrionfoInfo(
        variable="ii",
        title="ii",
        roman="II",
        name="L'Alta Sacerdotessa",
        puzzle="NEL DECIMO MESE",
        objective="Gioca almeno mezz'ora a [url=https://store.steampowered.com/app/881100]Noita[/url].",
        check=CheckPlayedSteamGame(881100),
    ),
    TrionfoInfo(
        variable="iii",
        title="iii",
        roman="III",
        name="L'Imperatrice",
        puzzle="RIMANI LIBERO",
        objective="Gioca almeno mezz'ora a [url=https://store.steampowered.com/app/245170]Skullgirls[/url].",
        check=CheckPlayedSteamGame(245170),
    ),
    TrionfoInfo(
        variable="iv",
        title="iv",
        roman="IV",
        name="L'Imperatore",
        puzzle="SEGUI GLI ORDINI",
        objective="Vinci una partita su [url=https://store.steampowered.com/app/611500]Quake Champions[/url].",
        check=CheckAchievementSteamGame(611500, "qc_victory")
    ),
    TrionfoInfo(
        variable="v",
        title="v",
        roman="V",
        name="Il Papa",
        puzzle="ALLA DECIMASESTA ORA",
        objective="Completa la prima zona di [url=https://store.steampowered.com/app/247080]Crypt of the Necrodancer["
                  "/url] con qualsiasi personaggio.",
        check=CheckAchievementSteamGame(247080, "ACH_ZONE1_COMPLETE")
    ),
    TrionfoInfo(
        variable="vi",
        title="vi",
        roman="VI",
        name="Gli Amanti",
        puzzle="ANCORA TRENTA MINUTI",
        objective="Gioca almeno mezz'ora a [url=https://store.steampowered.com/app/698780]Doki Doki Literature Club["
                  "/url].",
        check=CheckPlayedSteamGame(698780),
    ),
    TrionfoInfo(
        variable="vii",
        title="vii",
        roman="VII",
        name="Il Carro",
        puzzle="SOPRA UN CARRO",
        objective="Gioca 5 incontri di [url=https://store.steampowered.com/app/326460/ShellShock_Live]ShellShock Live["
                  "/url].",
        check=CheckAchievementSteamGame(326460, "play5")
    ),
    TrionfoInfo(
        variable="viii",
        title="viii",
        roman="VIII",
        name="La Giustizia",
        puzzle="RAGGIUNGI",
        objective="Porta la giustizia dalla tua parte su [url=https://store.steampowered.com/app/1289310]Helltaker["
                  "/url].",
        check=CheckAchievementSteamGame(1289310, "achiev_05"),
    ),
    TrionfoInfo(
        variable="ix",
        title="ix",
        roman="IX",
        name="L'Eremita",
        puzzle="SEGRETAMENTE",
        objective="Gioca almeno mezz'ora ad [url=https://store.steampowered.com/app/945360]Among Us[/url] su Steam.",
        check=CheckPlayedSteamGame(945360)
    ),
    TrionfoInfo(
        variable="x",
        title="x",
        roman="X",
        name="La Fortuna",
        puzzle="LA CASA DEI GIOCHI",
        objective="Chiedi a Royal Bot di predire il tuo futuro.",
        check=NullCheck(),
    ),
    TrionfoInfo(
        variable="xi",
        title="xi",
        roman="XI",
        name="La Forza",
        puzzle="ALLA LIBERA ROCCAFORTE",
        objective="Gioca almeno mezz'ora ad [url=https://store.steampowered.com/app/57300]Amnesia: The Dark Descent["
                  "/url].",
        check=CheckPlayedSteamGame(57300)
    ),
    TrionfoInfo(
        variable="xii",
        title="xii",
        roman="XII",
        name="L'Appeso",
        puzzle="PORTA CAVI E ARNESI",
        objective="Gioca almeno mezz'ora a [url=https://store.steampowered.com/app/381210]Dead by "
                  "Daylight.[/url]",
        check=CheckPlayedSteamGame(381210),
    ),
    TrionfoInfo(
        variable="xiii",
        title="xiii",
        roman="XIII",
        name="La Morte",
        puzzle="NON DIMENTICARE IL CIBO",
        objective="Gioca almeno mezz'ora a [url=https://store.steampowered.com/app/322330]Don't Starve Together[/url].",
        check=CheckPlayedSteamGame(322330),
    ),
    TrionfoInfo(
        variable="xiv",
        title="xiv",
        roman="XIV",
        name="La Temperanza",
        puzzle="NIENTE ALCOOL",
        objective="Raggiungi la Tenuta dell'Antenato su [url=https://store.steampowered.com/app/262060]Darkest Dungeon["
                  "/url].",
        check=CheckAchievementSteamGame(262060, "welcome_home"),
    ),
    TrionfoInfo(
        variable="xv",
        title="xv",
        roman="XV",
        name="Il Diavolo",
        puzzle="GIOCA CON LUI",
        objective="Completa la prima missione di [url=https://store.steampowered.com/app/379720]DOOM[/url].",
        check=CheckAchievementSteamGame(379720, "ach_4")
    ),
    TrionfoInfo(
        variable="xvi",
        title="xvi",
        roman="XVI",
        name="La Torre",
        puzzle="CONQUISTA LA CIMA",
        objective="Sconfiggi un boss del primo piano su [url=https://store.steampowered.com/app/646570/]"
                  "Slay the Spire[/url].",
        check=CheckAchievementSteamGame(646570, "GUARDIAN") or CheckAchievementSteamGame(646570, "GHOST_GUARDIAN") or
              CheckAchievementSteamGame(646570, "SLIME_BOSS")
    ),
    TrionfoInfo(
        variable="xvii",
        title="xvii",
        roman="XVII",
        name="Le Stelle",
        puzzle="STAI IN COMPAGNIA",
        objective="Completa due missioni co-op online su ["
                  "url=https://store.steampowered.com/app/630/Alien_Swarm]Alien Swarm[/url].",
        check=CheckAchievementSteamGame(630, "ASW_PARA_HAT"),
    ),
    TrionfoInfo(
        variable="xviii",
        title="xviii",
        roman="XVIII",
        name="La Luna",
        puzzle="FINO ALLA UNA",
        objective="Gioca almeno mezz'ora a [url=https://store.steampowered.com/app/388880]Oxenfree[/url].",
        check=CheckPlayedSteamGame(388880),
    ),
    TrionfoInfo(
        variable="xix",
        title="xix",
        roman="XIX",
        name="Il Sole",
        puzzle="IL TUO NOME SPLENDERÀ",
        objective="Gioca almeno mezz'ora a [url=https://store.steampowered.com/app/420530]OneShot[/url].",
        check=CheckPlayedSteamGame(420530),
    ),
    TrionfoInfo(
        variable="xx",
        title="xx",
        roman="XX",
        name="Il Giudizio",
        puzzle="E GIUDICHERÀ GLI ALTRI",
        objective="Completa la campagna Dead Center di [url=https://store.steampowered.com/app/550]"
                  "Left 4 Dead 2[/url].",
        check=CheckAchievementSteamGame(550, "ACH_SURVIVE_MALL"),
    ),
    TrionfoInfo(
        variable="xxi",
        title="xxi",
        roman="XXI",
        name="Il Mondo",
        puzzle="""44°35'45.0"N 11°02'58.9"E""",
        objective="Attraverso i Trionfi, il segreto sarà rivelato...",
        check=NullCheck(),
    ),
]


from typing import *
import royalnet
import royalnet.commands as rc
import random
import datetime


class AnswerCommand(rc.Command):
    name: str = "answer"

    description: str = "Fai una domanda al bot, che possa essere risposta con un sì o un no: lui ti risponderà!"

    syntax: str = ""

    _answers = [
        # Cerchiamo di tenere bilanciate le tre colonne, o almeno le prime due.
        # Se avete un'idea ma metterebbe troppe opzioni in un'unica categoria, mettetela sotto commento.

        # risposte "sì": 15
        "🔵 Sì.",
        "🔵 Decisamente sì!",
        "🔵 Uhm, secondo me sì.",
        "🔵 Sì! Sì! SÌ!",
        "🔵 Yup.",
        "🔵 Direi proprio di sì.",
        "🔵 Assolutamente sì.",
        "🔵 Ma certo!",
        "🔵 Esatto!",
        "🔵 Senz'altro!",
        "🔵 Ovviamente.",
        "🔵 Questa domanda ha risposta affermativa.",
        "🔵 Hell yeah.",
        "🔵 [url=https://www.youtube.com/watch?v=sq_Fm7qfRQk]YES! YES! YES![/url]",
        "🔵 yusssssss",

        # risposte "no": 15
        "❌ No.",
        "❌ Decisamente no!",
        "❌ Uhm, secondo me sì.",
        "❌ No, no, e ancora NO!",
        "❌ Nope.",
        "❌ Direi proprio di no.",
        "❌ Assolutamente no.",
        "❌ Certo che no!",
        "❌ Neanche per idea!",
        "❌ Neanche per sogno!",
        "❌ Niente affatto!",
        "❌ Questa domanda ha risposta negativa.",
        "❌ Hell no.",
        "❌ [url=https://www.youtube.com/watch?v=fKEZFRcuEqw]NO! NO! NO![/url]",
        "❌ lolno",

        # risposte "boh": 15
        "❔ Boh.",
        "❔ E io che ne so?!",
        "❔ Non so proprio rispondere.",
        "❔ Non lo so...",
        "❔ Mi avvalgo della facoltà di non rispondere.",
        "❔ Non parlerò senza il mio avvocato!",
        "❔ Dunno.",
        "❔ Perché lo chiedi a me?",
        "❔ Ah, non lo so io!",
        "❔ ¯\\_(ツ)_/¯",
        "❔ No idea.",
        "❔ Dunno.",
        "❔ Boooooh!",
        "❔ Non ne ho la più pallida idea.",
        "❔ No comment.",
        "❔ maibi",
    ]

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        h = hash(datetime.datetime.now())

        r = random.Random(x=h)

        message = r.sample(self._answers, 1)[0]
        await data.reply(message)



from typing import *
import royalnet
import royalnet.commands as rc
import royalnet.utils as ru
from ..tables import Treasure


class MagicktreasureCommand(rc.Command):
    name: str = "magicktreasure"

    description: str = "Crea un nuovo Treasure di Fiorygi (senza spendere i tuoi)."

    syntax: str = "{codice} {valore}"

    async def _permission_check(self, author, code, value, data, session):
        if "banker" not in author.roles:
            raise rc.UserError("Non hai permessi sufficienti per eseguire questo comando.")
        return author

    async def _create_treasure(self, author, code, value, data, session):
        TreasureT = self.alchemy.get(Treasure)

        treasure = await ru.asyncify(session.query(TreasureT).get, code)
        if treasure is not None:
            raise rc.UserError("Esiste già un Treasure con quel codice.")

        treasure = TreasureT(
            code=code,
            value=value,
            redeemed_by=None
        )

        return treasure

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        await data.delete_invoking()

        async with data.session_acm() as session:
            author = await data.find_author(session=session, required=True)

            code = args[0].lower()
            try:
                value = int(args[1])
            except ValueError:
                raise rc.InvalidInputError("Il valore deve essere maggiore o uguale a 0.")
            if value < 0:
                raise rc.InvalidInputError("Il valore deve essere maggiore o uguale a 0.")

            await self._permission_check(author, code, value, data, session)
            treasure = await self._create_treasure(author, code, value, data, session)
            session.add(treasure)
            await ru.asyncify(session.commit)

        await data.reply("✅ Treasure creato!")



from typing import *
import datetime

from sqlalchemy import *
from sqlalchemy.orm import *
from sqlalchemy.ext.declarative import declared_attr
import royalnet.utils as ru

from .fiorygi import Fiorygi

if TYPE_CHECKING:
    from royalnet.commands import CommandData


class FiorygiTransaction:
    __tablename__ = "fiorygitransactions"

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True)

    @declared_attr
    def change(self):
        return Column(Integer, nullable=False)

    @declared_attr
    def user_id(self):
        return Column(Integer, ForeignKey("fiorygi.user_id"), nullable=False)

    @declared_attr
    def wallet(self):
        return relationship("Fiorygi", backref=backref("transactions"))

    @property
    def user(self):
        return self.wallet.user

    @declared_attr
    def reason(self):
        return Column(String, nullable=False, default="")

    @declared_attr
    def timestamp(self):
        return Column(DateTime)

    def __repr__(self):
        return f"<{self.__class__.__name__}: {self.change:+} to {self.user.username} for {self.reason}>"

    @classmethod
    async def spawn_fiorygi(cls, user, qty: int, reason: str, *, data: "CommandData", session):
        FiorygiT = data.alchemy.get(cls)
        FiorygiTransactionT = data.alchemy.get(FiorygiTransaction)

        if user.fiorygi is None:
            session.add(
                FiorygiT(
                    user_id=user.uid,
                    fiorygi=0
                )
            )
            await ru.asyncify(session.commit)

        session.add(
            FiorygiTransactionT(
                user_id=user.uid,
                change=qty,
                reason=reason,
                timestamp=datetime.datetime.now()
            )
        )

        user.fiorygi.fiorygi += qty
        await ru.asyncify(session.commit)

        if len(user.telegram) > 0:
            user_str = user.telegram[0].mention()
        else:
            user_str = user.username

        if qty > 0:
            msg = f"💰 [b]{user_str}[/b] ha ottenuto [b]{qty}[/b] fioryg{'i' if qty != 1 else ''} per [i]{reason}[/i]!"
        elif qty == 0:
            msg = f"❓ [b]{user_str}[/b] ha mantenuto i suoi fiorygi attuali per [i]{reason}[/i].\nWait, cosa?"
        else:
            msg = f"💸 [b]{user_str}[/b] ha perso [b]{-qty}[/b] fioryg{'i' if qty != -1 else ''} per [i]{reason}[/i]."

        await data.command.serf.call_herald_event(
            "telegram", "telegram_message",
            chat_id=data.command.config["Telegram"]["main_group_id"],
            text=msg)



from typing import *
import logging
import royalnet.commands as rc
import royalnet.utils as ru
from royalnet.backpack import tables as rbt
from .abstract.linker import LinkerCommand
import asyncio
import datetime

from ..halloween2020 import *

log = logging.getLogger(__name__)


class TrionfirealiCommand(LinkerCommand):
    name: str = "trionfireali"

    description: str = "Visualizza il tuo pr⊕gress⊕ nei Tri⊕nfi Reali!"

    syntax: str = ""

    def describe(self, obj: TrionfiStatus) -> str:
        raise NotImplementedError()

    async def get_updatables_of_user(self, session, user: rbt.User) -> List[TrionfiStatus]:
        return [user.halloween2020] if user.halloween2020 else []

    async def get_updatables(self, session) -> List[TrionfiStatus]:
        return await ru.asyncify(session.query(self.alchemy.get(TrionfiStatus)).all)

    async def create(self,
                     session,
                     user: rbt.User,
                     args: rc.CommandArgs,
                     data: Optional[rc.CommandData] = None) -> Optional[TrionfiStatus]:
        raise rc.InvalidInputError("N⊕n è qui che inizia il mister⊕.")

    async def update(self, session, obj: TrionfiStatus, change: Callable[[str, Any], Awaitable[None]]):
        for trionfo in trionfilist:
            check_result = await trionfo.check.check(obj, key=self.config["steampowered"]["token"])
            log.debug(f"{trionfo.check}: {check_result}")
            if check_result is True:
                obj.__setattr__(trionfo.variable, datetime.datetime.now())
            await asyncio.sleep(1)

    async def on_increase(self, session, obj: TrionfiStatus, attribute: str, old: Any, new: Any) -> None:
        pass

    async def on_unchanged(self, session, obj: TrionfiStatus, attribute: str, old: Any, new: Any) -> None:
        pass

    async def on_decrease(self, session, obj: TrionfiStatus, attribute: str, old: Any, new: Any) -> None:
        pass

    async def on_first(self, session, obj: TrionfiStatus, attribute: str, old: None, new: Any) -> None:
        pass

    async def on_reset(self, session, obj: TrionfiStatus, attribute: str, old: Any, new: None) -> None:
        pass



import royalnet.utils as ru
from typing import *

if TYPE_CHECKING:
    from .check import Check


class TrionfoInfo:
    def __init__(self,
                 variable: str,
                 title: str,
                 roman: str,
                 name: str,
                 objective: str,
                 puzzle: str,
                 check: "Check"):
        self.variable: str = variable
        self.title: str = title
        self.roman: str = roman
        self.name: str = name
        self.objective: str = objective
        self.puzzle: str = puzzle
        self.check: "Check" = check

    def json_anonymous(self) -> ru.JSON:
        return {
            "variable": self.variable,
            "title": self.title,
            "roman": self.roman,
            "name": self.name,
            "objective": self.objective,
        }

    def json_user(self, obj) -> ru.JSON:
        status = obj.__getattribute__(self.variable)
        return {
            "variable": self.variable,
            "title": self.title,
            "roman": self.roman,
            "name": self.name,
            "objective": self.objective,
            "puzzle": self.puzzle if status is not None else None,
            "completed_on": status.timestamp() if status is not None else None
        }



from .check import *
from .trionfilist import *
from .trionfoinfo import *
from .trionfistatus import *

__all__ = [
    "Check",
    "CheckPlayedSteamGame",
    "CheckAchievementSteamGame",
    "TrionfiStatus",
    "TrionfoInfo",
    "trionfilist"
]



from typing import *
import royalnet.commands as rc
import royalnet.backpack.tables as rbt

from ..tables import FiorygiTransaction


class GivefiorygiCommand(rc.Command):
    name: str = "givefiorygi"

    description: str = "Cedi fiorygi a un altro utente."

    syntax: str = "{destinatario} {quantità} {motivo}"

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        async with data.session_acm() as session:
            author = await data.find_author(session=session, required=True)

            user_arg = args[0]
            qty_arg = args[1]

            if user_arg is None:
                raise rc.InvalidInputError("Non hai specificato un destinatario!")
            user = await rbt.User.find(alchemy=self.alchemy, session=session, identifier=user_arg)
            if user is None:
                raise rc.InvalidInputError("L'utente specificato non esiste!")
            if user.uid == author.uid:
                raise rc.InvalidInputError("Non puoi inviare fiorygi a te stesso!")

            if qty_arg is None:
                raise rc.InvalidInputError("Non hai specificato una quantità!")
            try:
                qty = int(qty_arg)
            except ValueError:
                raise rc.InvalidInputError("La quantità specificata non è un numero!")
            if qty <= 0:
                raise rc.InvalidInputError("La quantità specificata deve essere almeno 1!")

            if author.fiorygi.fiorygi < qty:
                raise rc.InvalidInputError("Non hai abbastanza fiorygi per effettuare la transazione!")

            await FiorygiTransaction.spawn_fiorygi(author, -qty, f"aver ceduto fiorygi a {user}",
                                                   data=data,
                                                   session=session)
            await FiorygiTransaction.spawn_fiorygi(user, qty, f"aver ricevuto fiorygi da {author}",
                                                   data=data,
                                                   session=session)



import datetime
import royalnet.backpack.tables as rbt
import royalnet.constellation.api as rca
from .trionfistatus import TrionfiStatus


class ApiUserTrionfiStar(rca.ApiStar):
    path = "/api/user/trionfi/v2"

    parameters = {
        "get": {
            "uid": "(Choose one) The id of the user to get information about.",
            "alias": "(Choose one) The alias of the user to get information about.",
        }
    }

    auth = {
        "get": True
    }

    tags = ["user"]

    async def get_user(self, data: rca.ApiData):
        uid = data.int("uid", optional=True)
        alias = data.str("alias", optional=True)

        if uid:
            user = await rbt.User.find(self.alchemy, data.session, uid)
        elif alias:
            user = await rbt.User.find(self.alchemy, data.session, alias)
        else:
            raise rca.MissingParameterError("Neither uid or alias were specified.")

        if user is None:
            raise rca.NotFoundError("No such user.")

        return user

    @rca.magic
    async def get(self, data: rca.ApiData) -> dict:
        """Get Royalpack information about a user."""
        author = await data.user()
        if len(author.steam) >= 1:
            steam = author.steam[0]
            if steam.trionfistatus is None:
                TrionfiStatusT = self.alchemy.get(TrionfiStatus)
                ts = TrionfiStatusT(steam=steam)
                data.session.add(ts)
                await data.session_commit()
            if not steam.trionfistatus.zero:
                steam.trionfistatus.zero = datetime.datetime.now()
                await data.session_commit()

        user = await self.get_user(data)
        result = {
            **user.json(),
            "bio": user.bio.json() if user.bio is not None else None,
            "fiorygi": user.fiorygi.fiorygi if user.fiorygi is not None else None,
            "steam": [steam.json() for steam in user.steam],
            "leagueoflegends": [leagueoflegends.json() for leagueoflegends in user.leagueoflegends],
            "osu": [osu.json() for osu in user.osu],
            "trivia": user.trivia_score.json() if user.trivia_score is not None else None,
        }
        return result



from typing import *
import asyncio
import aiohttp
import random
import uuid
import html
import royalnet.commands as rc
import royalnet.utils as ru
import royalnet.backpack.tables as rbt
from ..tables import TriviaScore


class TriviaCommand(rc.Command):
    name: str = "trivia"

    aliases = ["t"]

    description: str = "Manda una domanda dell'OpenTDB in chat."

    syntax = "[credits|scores]"

    _letter_emojis = ["🇦", "🇧", "🇨", "🇩"]

    _medal_emojis = ["🥇", "🥈", "🥉", "🔹"]

    _correct_emoji = "✅"

    _wrong_emoji = "❌"

    _answer_time = 20

    # _question_lock: bool = False

    def __init__(self, serf, config):
        super().__init__(serf, config)
        self._answerers: Dict[uuid.UUID, Dict[str, bool]] = {}

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        arg = args.optional(0)
        async with data.session_acm() as session:
            if arg == "credits":
                await data.reply(f"ℹ️ [c]{self.serf.prefix}{self.name}[/c] di [i]Steffo[/i]\n"
                                 f"\n"
                                 f"Tutte le domande vengono dall'[b]Open Trivia Database[/b] di [i]Pixeltail Games[/i],"
                                 f" creatori di Tower Unite, e sono rilasciate sotto la licenza [b]CC BY-SA 4.0[/b].")
                return
            elif arg == "scores":
                trivia_scores = await ru.asyncify(session.query(self.alchemy.get(TriviaScore)).all)
                strings = ["🏆 [b]Trivia Leaderboards[/b]\n"]
                for index, ts in enumerate(sorted(trivia_scores, key=lambda ts: -ts.score)):
                    if index > 3:
                        index = 3
                    strings.append(f"{self._medal_emojis[index]} {ts.user.username}: [b]{ts.score:.0f}p[/b]"
                                   f" ({ts.correct_answers}/{ts.total_answers})")
                await data.reply("\n".join(strings))
                return
            # if self._question_lock:
            #     raise rc.CommandError("C'è già un'altra domanda attiva!")
            # self._question_lock = True
            # Fetch the question
            async with aiohttp.ClientSession() as ws:
                async with ws.get("https://opentdb.com/api.php?amount=1") as response:
                    j = await response.json()
            # Parse the question
            if j["response_code"] != 0:
                raise rc.CommandError(f"OpenTDB returned an error response_code ({j['response_code']}).")
            question = j["results"][0]
            text = f'❓ [b]{question["category"]}[/b]\n' \
                   f'{html.unescape(question["question"])}'
            # Prepare answers
            correct_answer: str = question["correct_answer"]
            wrong_answers: List[str] = question["incorrect_answers"]
            answers: List[str] = [correct_answer, *wrong_answers]
            if question["type"] == "multiple":
                random.shuffle(answers)
            elif question["type"] == "boolean":
                answers.sort(key=lambda a: a)
                answers.reverse()
            else:
                raise NotImplementedError("Unknown question type")
            # Find the correct index
            for index, answer in enumerate(answers):
                if answer == correct_answer:
                    correct_index = index
                    break
            else:
                raise ValueError("correct_index not found")
            # Add emojis
            for index, answer in enumerate(answers):
                answers[index] = f"{self._letter_emojis[index]} {html.unescape(answers[index])}"
            # Create the question id
            question_id = uuid.uuid4()
            self._answerers[question_id] = {}

            # Create the correct and wrong functions
            async def correct(data: rc.CommandData):
                answerer_ = await data.find_author(session=session, required=True)
                try:
                    self._answerers[question_id][answerer_.uid] = True
                except KeyError:
                    raise rc.UserError("Tempo scaduto!")
                await data.reply("🆗 Hai risposto alla domanda. Ora aspetta un attimo per i risultati!")

            async def wrong(data: rc.CommandData):
                answerer_ = await data.find_author(session=session, required=True)
                try:
                    self._answerers[question_id][answerer_.uid] = False
                except KeyError:
                    raise rc.UserError("Tempo scaduto!")
                await data.reply("🆗 Hai risposto alla domanda. Ora aspetta un attimo per i risultati!")

            # Add question
            keyboard: List[rc.KeyboardKey] = []
            for index, answer in enumerate(answers):
                if index == correct_index:
                    keyboard.append(rc.KeyboardKey(short=self._letter_emojis[index],
                                                   text=answers[index],
                                                   callback=correct))
                else:
                    keyboard.append(rc.KeyboardKey(short=self._letter_emojis[index],
                                                   text=answers[index],
                                                   callback=wrong))
            async with data.keyboard(text=text, keys=keyboard):
                await asyncio.sleep(self._answer_time)
            results = f"❗️ Tempo scaduto!\n" \
                      f"La risposta corretta era [b]{answers[correct_index]}[/b]!\n\n"
            for answerer_id in self._answerers[question_id]:
                answerer = session.query(self.alchemy.get(rbt.users.User)).get(answerer_id)
                if answerer.trivia_score is None:
                    ts = self.alchemy.get(TriviaScore)(user=answerer)
                    session.add(ts)
                    await ru.asyncify(session.commit)
                previous_score = answerer.trivia_score.score
                if self._answerers[question_id][answerer_id]:
                    results += self._correct_emoji
                    answerer.trivia_score.correct_answers += 1
                else:
                    results += self._wrong_emoji
                    answerer.trivia_score.wrong_answers += 1
                current_score = answerer.trivia_score.score
                score_difference = current_score - previous_score
                results += f" {answerer}: [b]{current_score:.0f}p[/b] ({score_difference:+.0f}p)\n"
            await data.reply(results)
            del self._answerers[question_id]
            await ru.asyncify(session.commit)
            # self._question_lock = False



from typing import *
import royalnet
import royalnet.commands as rc
import royalnet.utils as ru
from ..tables import Treasure, FiorygiTransaction


class TreasureCommand(rc.Command):
    name: str = "treasure"

    description: str = "Riscatta un Treasure che hai trovato da qualche parte."

    syntax: str = "{code}"

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        TreasureT = self.alchemy.get(Treasure)

        async with data.session_acm() as session:
            author = await data.find_author(session=session, required=True)
            code = args[0].lower()

            treasure = await ru.asyncify(session.query(TreasureT).get, code)
            if treasure is None:
                raise rc.UserError("Non esiste nessun Treasure con quel codice.")
            if treasure.redeemed_by is not None:
                raise rc.UserError(f"Quel tesoro è già stato riscattato da {treasure.redeemed_by}.")

            treasure.redeemed_by = author
            await ru.asyncify(session.commit)
            await FiorygiTransaction.spawn_fiorygi(author,
                                                   treasure.value,
                                                   f'aver trovato il tesoro "{treasure.code}"',
                                                   data=data,
                                                   session=session)
            await data.reply("🤑 Tesoro riscattato!")



from sqlalchemy import *
from sqlalchemy.orm import relationship, backref
from sqlalchemy.ext.declarative import declared_attr
from .trionfilist import trionfilist


class TrionfiStatus:
    __tablename__ = "trionfistatus"

    @declared_attr
    def _steamid(self):
        return Column(BigInteger, ForeignKey("steam._steamid"), primary_key=True)

    @declared_attr
    def steam(self):
        return relationship("Steam", backref=backref("trionfistatus", uselist=False))

    @declared_attr
    def zero(self):
        return Column(DateTime)

    @declared_attr
    def i(self):
        return Column(DateTime)

    @declared_attr
    def ii(self):
        return Column(DateTime)

    @declared_attr
    def iii(self):
        return Column(DateTime)

    @declared_attr
    def iv(self):
        return Column(DateTime)

    @declared_attr
    def v(self):
        return Column(DateTime)

    @declared_attr
    def vi(self):
        return Column(DateTime)

    @declared_attr
    def vii(self):
        return Column(DateTime)

    @declared_attr
    def viii(self):
        return Column(DateTime)

    @declared_attr
    def ix(self):
        return Column(DateTime)

    @declared_attr
    def x(self):
        return Column(DateTime)

    @declared_attr
    def xi(self):
        return Column(DateTime)

    @declared_attr
    def xii(self):
        return Column(DateTime)

    @declared_attr
    def xiii(self):
        return Column(DateTime)

    @declared_attr
    def xiv(self):
        return Column(DateTime)

    @declared_attr
    def xv(self):
        return Column(DateTime)

    @declared_attr
    def xvi(self):
        return Column(DateTime)

    @declared_attr
    def xvii(self):
        return Column(DateTime)

    @declared_attr
    def xviii(self):
        return Column(DateTime)

    @declared_attr
    def xix(self):
        return Column(DateTime)

    @declared_attr
    def xx(self):
        return Column(DateTime)

    @declared_attr
    def xxi(self):
        return Column(DateTime)

    def total(self):
        return sum(map(lambda i: 0 if i is None else 1, [
            self.zero,
            self.i,
            self.ii,
            self.iii,
            self.iv,
            self.v,
            self.vi,
            self.vii,
            self.viii,
            self.ix,
            self.x,
            self.xi,
            self.xii,
            self.xiii,
            self.xiv,
            self.xv,
            self.xvi,
            self.xvii,
            self.xviii,
            self.xix,
            self.xx,
            self.xxi,
        ]))

    def json(self):
        return {
            "completed": self.total(),
            "trionfi": {trionfo.variable: trionfo.json_user(self) for trionfo in trionfilist}
        }



from typing import *
import abc
import aiohttp
import logging
import royalnet.commands as rc

if TYPE_CHECKING:
    from .trionfistatus import TrionfiStatus


log = logging.getLogger(__name__)


__all__ = [
    "Check",
    "NullCheck",
    "CheckPlayedSteamGame",
    "CheckAchievementSteamGame",
]


class Check(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    async def check(self, status: "TrionfiStatus", key: str) -> bool:
        raise NotImplementedError()

    def __or__(self, other: "Check"):
        return CheckOr(self, other)

    def __and__(self, other):
        return CheckAnd(self, other)


class NullCheck(Check):
    def __repr__(self):
        return f"{self.__class__.__name__}()"

    async def check(self, status: "TrionfiStatus", key: str) -> bool:
        return False


class CheckPlayedSteamGame(Check):
    def __init__(self, appid: int, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.appid: int = appid

    def __repr__(self):
        return f"{self.__class__.__name__}({self.appid=})"

    async def check(self, status: "TrionfiStatus", key: str) -> bool:
        log.debug(f"{self}")
        async with aiohttp.ClientSession() as ah_session:
            # noinspection PyProtectedMember
            async with ah_session.get("https://api.steampowered.com/IPlayerService/GetOwnedGames/v1/",
                                      params={
                                          "steamid": status._steamid,
                                          "include_appinfo": "true",
                                          "include_played_free_games": "true",
                                          "include_free_sub": "true",
                                          "appids_filter": self.appid,
                                          "key": key,
                                      }) as response:
                try:
                    j = await response.json()
                except Exception as e:
                    log.error(f"{e}")
                    return False

                games = j["response"]["games"]
                for game in games:
                    if game["appid"] != self.appid:
                        continue
                    if game["playtime_forever"] >= 30:
                        return True
                return False


class CheckAchievementSteamGame(Check):
    def __init__(self, appid: int, achievement_name: str, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.appid: int = appid
        self.achivement_name: str = achievement_name

    def __repr__(self):
        return f"{self.__class__.__name__}({self.appid=}, {self.achivement_name=})"

    async def check(self, status: "TrionfiStatus", key: str) -> bool:
        log.debug(f"{self}")
        async with aiohttp.ClientSession() as ah_session:
            # noinspection PyProtectedMember
            async with ah_session.get("http://api.steampowered.com/ISteamUserStats/GetPlayerAchievements/v1/",
                                      params={
                                          "steamid": status._steamid,
                                          "appid": self.appid,
                                          "key": key,
                                      }) as response:
                try:
                    j = await response.json()
                except Exception as e:
                    log.error(f"{e}")
                    return False
                if not j["playerstats"]["success"]:
                    log.warning(f"{j}")
                    return False

                achievements = j["playerstats"]["achievements"]
                for ach in achievements:
                    if ach["apiname"] != self.achivement_name:
                        continue
                    return ach["achieved"] == 1
                return False


class CheckOr(Check):
    def __init__(self, first: Check, second: Check, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.first: Check = first
        self.second: Check = second

    def __repr__(self):
        return f"{self.first} or {self.second}"

    async def check(self, status: "TrionfiStatus", key: str) -> bool:
        log.debug(f"{self}")
        return (await self.first.check(status, key)) or (await self.second.check(status, key))


class CheckAnd(Check):
    def __init__(self, first: Check, second: Check, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.first: Check = first
        self.second: Check = second

    def __repr__(self):
        return f"{self.first} and {self.second}"

    async def check(self, status: "TrionfiStatus", key: str) -> bool:
        log.debug(f"{self}")
        return (await self.first.check(status, key)) and (await self.second.check(status, key))



from typing import *

import functools
import pkg_resources
import royalnet.commands as rc


class RoyalpackCommand(rc.Command):
    name: str = "royalpackversion"

    description: str = "Visualizza la versione attuale di Royalpack."

    syntax: str = ""

    @property
    @functools.lru_cache(1)
    def royalpack_version(self) -> str:
        return pkg_resources.get_distribution("royalpack").version

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        message = f"ℹ️ Royalpack [url=https://github.com/Steffo99/royalpack/releases/tag/{self.royalpack_version}]{self.royalpack_version}[/url]\n"
        if "69" in self.royalpack_version:
            message += "(Nice.)"
        await data.reply(message)



from typing import *
import royalnet.commands as rc
import royalnet.backpack.tables as rbt


class EvalCommand(rc.Command):
    # oh god if there is a security vulnerability
    name: str = "eval"

    description: str = "Esegui una espressione Python... se sei Steffo."

    syntax: str = "{espressione}"

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        async with data.session_acm() as session:
            user: rbt.User = await data.find_author(session=session, required=True)
            if "admin" not in user.roles:
                raise rc.CommandError("Non sei autorizzato a eseguire codice arbitrario!\n"
                                      "(Sarebbe un po' pericoloso se te lo lasciassi eseguire, non trovi?)")
        try:
            result = eval(args.joined(require_at_least=1))
        except Exception as e:
            raise rc.CommandError(f"Eval fallito: {e}")
        await data.reply(repr(result))



from typing import *
import royalnet
import royalnet.commands as rc
import royalnet.utils as ru
import random
import datetime


class FortuneCommand(rc.Command):
    name: str = "fortune"

    description: str = "Quanto sarai fortunato oggi?"

    syntax: str = ""

    _fortunes = [
        "😄 Oggi sarà una fantastica giornata!",
        "😌 Oggi sarà una giornata molto chill e rilassante.",
        "💰 Oggi sui tuoi alberi cresceranno più Stelline!",
        "🍎 Oggi un unicorno ti lascerà la sua Blessed Apple!",
        "📈 Oggi il tuo team in ranked sarà più amichevole e competente del solito!",
        "🏝 Oggi potrai raggiungere l'Isola Miraggio!",
        "🐱 Oggi vedrai più gatti del solito su Internet!",
        "🐶 Oggi vedrai più cani del solito su Internet!",
        "🐦 Oggi vedrai più uccelli del solito su Internet!",
        "🔥 Oggi vedrai più flame del solito su Internet!",
        "🤬 Oggi vedrai più discorsi politici del solito su Internet!",
        "🐌 Oggi incontrerai una chiocciola sperduta!",
        "🎁 Oggi i dispenser di regali in centro funzioneranno senza problemi!",
        "🥕 Oggi il tuo raccolto avrà qualità Iridium Star!",
        "🔴 Oggi troverai più oggetti di rarità rossa del solito!",
        "✨ Oggi farai molti più multicast!",
        "♦️ Oggi troverai una Leggendaria Dorata!",
        "⭐️ Oggi la stella della RYG ti sembrerà un pochino più dritta!",
        "⭐️ Oggi la stella della RYG ti sembrerà anche più storta del solito!",
        "💎 Oggi i tuoi avversari non riusciranno a deflettere i tuoi Emerald Splash!",
        "⬅️ Oggi le tue supercazzole prematureranno un po' più a sinistra!",
        "➡️ Oggi le tue supercazzole prematureranno un po' più a destra!",
        "🌅 Oggi sarà il giorno dopo ieri e il giorno prima di domani!",
        "🤖 Oggi il Royal Bot ti dirà qualcosa di molto utile!",
        "💤 Oggi rischierai di addormentarti più volte!",
        "🥪 Oggi ti verrà fame fuori orario!",
        "😓 Oggi dirai molte stupidaggini!",
        "🏠 Oggi qualcuno si autoinviterà a casa tua!",
        "📵 Oggi passerai una bella giornata tranquilla senza che nessuno ti chiami!",
        "🎶 Oggi scoprirai un vero [url=https://www.urbandictionary.com/define.php?term=banger]banger[/url]!",
        "🕸 Oggi cadrai trappola di una ragnatela! [i]O ti arriverà in faccia.[/i]",
        "🧻 Oggi fai attenzione alla carta igienica: potrebbe finire!",
        "🔮 Oggi chiederai a @royalgamesbot di dirti la tua /fortune!",
        "🧨 Oggi calpesterai delle [url=https://www.youtube.com/watch?v=Zyef3NU3wqk&t=57]mine di Techies[/url]!",
        "👽 Oggi incontrerai gli UFI!!!1!!uno!",
        "🦾 Oggi uno scienziato pazzo ti proporrà di sostituire il tuo braccio con un braccio-razzo meccanico!",
        "🕵️ Oggi una spia in incognito ti chiederà se hai mai visto the Emoji Movie!",
        "🍕 Oggi mangerai una margherita doppio pomodoro!",
        "🍰 Oggi mangerai una [url=https://www.youtube.com/watch?v=2EWWL3niBWY]torta al gusto di torta[/url]!",
        "🥇 Oggi vincerai qualcosa!",
        "🏴‍☠️ Oggi salperai i sette mari con la tua ciurma pirata!",
        "🕒 Oggi sarà ieri, e domani sarà oggi!",
        "🔙 Oggi torneai indietro nel tempo!",
        "🚨 Oggi suonerà l'allarme della Velvet Room!",
        "♾ Oggi ti sembrerà infinito!",
        "👹 Oggi perfino i demoni avranno paura di te!",
        "🥐 Oggi mangerai una brioche (o lancerai un boomerang)!",
    ]

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        async with data.session_acm() as session:
            author = await data.find_author(session=session, required=True)
            today = datetime.date.today()

            h = author.uid * hash(today)

            r = random.Random(x=h)

            message = r.sample(self._fortunes, 1)[0]
            await data.reply(message)




from typing import *
import royalnet.commands as rc
import royalnet.utils as ru
from sqlalchemy import func

from ..tables import Diario


class DiarioshuffleCommand(rc.Command):
    name: str = "diarioshuffle"

    description: str = "Cita una riga casuale del diario."

    aliases = ["dis", "dishuffle", "dish"]

    syntax = ""

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        async with data.session_acm() as session:
            DiarioT = self.alchemy.get(Diario)
            entry: List[Diario] = await ru.asyncify(
                session.query(DiarioT).order_by(func.random()).limit(1).one_or_none
            )
            if entry is None:
                raise rc.CommandError("Nessuna riga del diario trovata.")
            await data.reply(f"ℹ️ {entry}")
